import os
import logging


scale_percent = 40  # percent of original size
predict_img_size = 28
# mark
mark_size = 16
correct_mark_color = (0, 200, 0)#(176, 255, 80)
wrong_mark_color = (0, 0, 200)
mark_thickness = 2
DONT_SHOW = True
BASE_DIR = os.path.dirname(os.path.abspath(__file__))

log_path = os.path.join(BASE_DIR, '../logs/app.log')
logging.basicConfig(filename=log_path, filemode='w', format='%(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)



import numpy as np
import cv2
import imutils
from .setting import DONT_SHOW, logger
import time



def find_contours(grayImageBlur):
    v = np.median(grayImageBlur)
    sigma = 0.5
    lower = int(max(0, (1.0 - sigma) * v))
    upper = int(min(255, (1.0 + sigma) * v))
    # теперь производим определение границы по методу Canny
    edgedImage = cv2.Canny(grayImageBlur, lower, upper, 3)
    # найти контуры на обрезанном изображении, рационально организовать область
    # оставить только большие варианты
    allContours = cv2.findContours(edgedImage.copy(), cv2.RETR_LIST, cv2.CHAIN_APPROX_SIMPLE)
    allContours = imutils.grab_contours(allContours)
    return allContours


def find_dimensions(allContours):
    # сортировка контуров области по уменьшению и сохранение топ-1
    allContours = sorted(allContours, key=cv2.contourArea, reverse=True)
    all_dimensions = []
    for contour in allContours:
        # aппроксимация контура
        perimeter = cv2.arcLength(contour, True)
        ROIdimensions = cv2.approxPolyDP(contour, 0.02 * perimeter, True)

        # изменение массива координат
        if ROIdimensions.shape == (4, 1, 2):
            ROIdimensions = ROIdimensions.reshape(4, 2)
            all_dimensions.append(ROIdimensions)
    return all_dimensions


def chose_contours(all_dimensions, img_high, img_weight, blank_type):
    try:
        table_1 = all_dimensions[0]
        table_2 = all_dimensions[1]
        if blank_type[0] == 'v':
            for i in range(len(all_dimensions)):
                contour_mid_h = (all_dimensions[i][2][1] + all_dimensions[i][3][1]) / 2
                if img_high * 0.2 < contour_mid_h and contour_mid_h < img_high * 0.6:
                    table_1 = all_dimensions[i]
                    break
        elif blank_type[0] == 'h':
            for i in range(len(all_dimensions)):
                contour_mid_w = (all_dimensions[i][2][0] + all_dimensions[i][3][0]) / 2
                if contour_mid_w > img_weight * 0.5:
                    table_1 = all_dimensions[i]
                    break
        elif blank_type[0] == 'q':
            for i in range(len(all_dimensions)):
                contour_mid_w = (all_dimensions[i][2][0] + all_dimensions[i][3][0]) / 2
                if contour_mid_w > img_weight * 0.5:
                    table_1 = all_dimensions[i]
                    break
        else:
            logger.warning('не удалось опознать тип бланка')
        table_1_right_side = sorted(table_1, key=lambda x: x[0])[2:]
        table_1_right_bottom = sorted(table_1_right_side, key=lambda x: x[1])[1][1]
        for i in range(len(all_dimensions)):
            right_side = sorted(all_dimensions[i], key=lambda x: x[0])[2:]
            right_top = sorted(right_side, key=lambda x: x[1])[0][1]
            if right_top > table_1_right_bottom:
                table_2 = all_dimensions[i]
                break
        return [table_1, table_2]
    except Exception as ex:
        logger.warning(f'Ошибка при выборе контуров таблиц: {ex}')



def find_coordinates(ROIdimensions):
    rect = np.zeros((4, 2), dtype="float32")

    s = np.sum(ROIdimensions, axis=1)

    rect[0] = ROIdimensions[np.argmin(s)]
    rect[2] = ROIdimensions[np.argmax(s)]

    diff = np.diff(ROIdimensions, axis=1)
    rect[1] = ROIdimensions[np.argmin(diff)]
    rect[3] = ROIdimensions[np.argmax(diff)]
    return rect


def find_dist(x1, x2, y1, y2):
    dist = np.sqrt((x1 - x2) ** 2 + (y1 - y2) ** 2)
    return dist


def find_max_width(rect):
    # верх-лево, верх-право, низ-право, низ-лево
    (tl, tr, br, bl) = rect

    # вычислить ширину ROI
    widthA = find_dist(tl[0], tr[0], tl[1], tr[1])
    widthB = find_dist(bl[0], br[0], bl[1], br[1])
    maxWidth = max(int(widthA), int(widthB))

    # вычислить высоту ROI
    heightA = find_dist(tl[0], bl[0], tl[1], bl[1])
    heightB = find_dist(tr[0], br[0], tr[1], br[1])
    maxHeight = max(int(heightA), int(heightB))

    return maxWidth, maxHeight


def transform_img(grayImage, rect, maxWidth, maxHeight):
    # набор итоговых точек для обзора всего документа
    # размер нового изображения
    dst = np.array([
        [0, 0],
        [maxWidth - 1, 0],
        [maxWidth - 1, maxHeight - 1],
        [0, maxHeight - 1]], dtype="float32")

    # вычислить матрицу перспективного преобразования и применить её
    transformMatrix = cv2.getPerspectiveTransform(rect, dst)
    # преобразовать ROI
    scan = cv2.warpPerspective(grayImage, transformMatrix, (maxWidth, maxHeight))
    return scan, transformMatrix

def take_it(image, blank_type):
    try:
        start_time = time.time()
        grayImage = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
        # размытие картинки, чтобы убрать высокочастотный шум
        # это помогает определить контур в сером изображении
        grayImageBlur = cv2.blur(grayImage, (3, 3))
        allContours = find_contours(grayImageBlur)
        all_dimensions = find_dimensions(allContours)
        img_high = image.shape[0]
        img_weight = image.shape[1]
        all_rect = []
        for dim in all_dimensions:
            rect = find_coordinates(dim)
            all_rect.append(rect)
        all_dimensions = chose_contours(all_dimensions, img_high, img_weight, blank_type)
        rects = []
        scans = []
        for ROIdimensions in all_dimensions:
            rect = find_coordinates(ROIdimensions)

            maxWidth, maxHeight = find_max_width(rect)

            scan, t_matrix = transform_img(grayImage, rect, maxWidth, maxHeight)
            cv2.drawContours(image, [ROIdimensions], -1, (0, 255, 0), 2)
            rects.append(rect)
            scans.append(scan)
        if not DONT_SHOW:
            cv2.imshow('Contour Outline', image)
            print("--- %s seconds ---" % (time.time() - start_time))

        return image, scans, rects
    except Exception as ex:
        logger.warning(f'Ошибка при поиске таблиц задач: {ex}')


import numpy as np
import cv2
from .predict import predict_img
from .preprocessing import make_contrast
from .setting import DONT_SHOW, logger
import time


def crop_results_v(image, num, blank_type):
    try:
        w = image.shape[1]
        h = image.shape[0]
        y0 = 23 * h // 32
        y1 = 28 * h // 32
        x0 = (31 + 80 * num) * w / 832
        x1 = (31 + 80 * (num + 1)) * w / 832
        if blank_type[1:] == '6x10':
            y0 = 47 * h // 64
            y1 = 59 * h // 64
        elif blank_type[1:] == '7x10':
            y0 = 49 * h // 64
            y1 = 59 * h // 64
        elif blank_type[1:] == '8x10':
            y0 = 51 * h // 64
            y1 = 59 * h // 64
        elif blank_type[1:] == '9x10':
            y0 = 52 * h // 64
            y1 = 60 * h // 64
        elif blank_type[1:] == '10x10':
            y0 = 53 * h // 64
            y1 = 60 * h // 64
        tetta = 0.1
        zetta = (x1 - x0) * tetta
        x0, x1 = int(x0 - zetta), int(x1 + zetta)
        gretta = 1
        crop_img = image[y0 - gretta:y1 + gretta, x0:x1]
        if not DONT_SHOW:
            cv2.imwrite(f'write/v_0crop_{str(int(time.time() * 1000))}.jpg', crop_img)
            cv2.imshow('crop_mrop', image)
        return crop_img
    except Exception as ex:
        logger.warning(f'Ошибка при вырезании одного ответа: {ex}')

def crop_results_h(image, num, blank_type):
    try:
        w = image.shape[1]
        h = image.shape[0]
        y0 = 23 * h // 32
        y1 = 28 * h // 32
        if blank_type[1:] == '6x10':
            y0 = 24 * h // 32
            y1 = 29 * h // 32
        tetta = 0.05
        x0 = (31 + 80 * num) * w / 832
        x1 = (31 + 80 * (num + 1)) * w / 832
        zetta = (x1 - x0) * tetta
        x0, x1 = int(x0 - zetta), int(x1 + zetta)
        crop_img = image[y0:y1, x0:x1]
        if not DONT_SHOW:
            image = cv2.rectangle(image, (x0, y0), (x1, y1), (255, 255, 255), 1)
            cv2.imshow('crop_mrop', image)
        return crop_img
    except Exception as ex:
        logger.warning(f'Ошибка при вырезании одного ответа: {ex}')


def crop_three(contrast_scan, image):
    try:
        g = 0
        im2 = contrast_scan
        img_w = image.shape[1]
        img_h = image.shape[0]
        contours, hierarchy = cv2.findContours(image, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
        all_rects = []
        for i in range(len(contours)):
            cnt = contours[i]
            im = im2.copy()
            im = cv2.cvtColor(im, cv2.COLOR_GRAY2BGR)
            cv2.drawContours(im,[cnt],0,255,-1)
            cv2.imwrite(f'write/z_{time.time()}_' + str(i) + '.png', im)
            x,y,w,h = cv2.boundingRect(cnt)
            if w<h*1.4 and h<w*1.4 and w > 0.25 * img_w and h > 0.6 * img_h:
                all_rects.append([x,y,w,h])

        all_rects = sorted(all_rects, key=lambda x: x[-1] * x[-2])[-3:]
        all_rects = sorted(all_rects, key=lambda x: x[0])
        while len(all_rects) < 3:
            all_rects.append([0, 0, 0, 0])
        x,y,w,h = all_rects[0][:4]
        if not(0.05 * img_w < x and x < 0.15 * img_w):
            x = int(0.1 * img_w)
        if not (0.13 * img_h < y < 0.22 * img_h):
            y = int(0.19 * img_h)
        if not(0.2 * img_w < w < 0.3 * img_w):
            w = int(0.26 * img_w)
        if not (0.55 * img_h < h < 0.74 * img_h):
            h = int(0.64 * img_h)
        image1 = im2[y+ g:y + h -g,x+g:x+w-g]

        x,y,w,h = all_rects[1][:4]
        if not(0.32 * img_w < x < 0.42 * img_w):
            x = int(0.36 * img_w)
        if not (0.13 * img_h < y < 0.22 * img_h):
            y = int(0.19 * img_h)
        if not(0.2 * img_w < w < 0.3 * img_w):
            w = int(0.26 * img_w)
        if not (0.55 * img_h < h < 0.74 * img_h):
            h = int(0.64 * img_h)
        image2 = im2[y+ g:y + h -g,x+g:x+w-g]

        x,y,w,h = all_rects[2][:4]
        if not(0.6 * img_w < x < 0.72 * img_w):
            x = int(0.67 * img_w)
        if not (0.13 * img_h < y < 0.22 * img_h):
            y = int(0.19 * img_h)
        if not(0.22 * img_w < w < 0.32 * img_w):
            w = int(0.28 * img_w)
        if not (0.55 * img_h < h < 0.74 * img_h):
            h = int(0.64 * img_h)
        image3 = im2[y+ g:y + h -g,x+g:x+w-g]
        return image1, image2, image3
    except Exception as ex:
        logger.warning(f'Ошибка при вырезании одной цифры ответа: {ex}')



def make_it(scans, blank_type):
    try:
        table_1, table_2 = scans
        result = []
        for table in [table_1, table_2]:
            table_result = []
            for num in range(10):
                if not DONT_SHOW:
                    print(num)
                try:
                    if blank_type[0] == 'v':
                        scan = crop_results_v(table, num, blank_type)
                    else:
                        scan = crop_results_h(table, num, blank_type)
                    bg_img = cv2.medianBlur(scan, 25)
                    diff_img = 255 - cv2.absdiff(scan, bg_img)
                    norm_img = cv2.normalize(diff_img, None, alpha=0, beta=255, norm_type=cv2.NORM_MINMAX,
                                             dtype=cv2.CV_8UC1)
                    if not DONT_SHOW:
                        cv2.imshow('norm_img', cv2.resize(norm_img, (0, 0), fx=3.5, fy=3.5))
                        cv2.imwrite(f'write/aaaaaa{num}.png', norm_img)

                    contrast_scan = make_contrast(scan)
                    contrast_img = make_contrast(norm_img)
                    croped_one, croped_two, croped_three = crop_three(contrast_scan, contrast_img)
                    result_one = predict_img(croped_one)
                    result_two = predict_img(croped_two)
                    result_three = predict_img(croped_three)
                    result_text = result_one + result_two + result_three
                    table_result.append(result_text)
                except:
                    table_result.append('error')
                    if not DONT_SHOW:
                        print('Не удалось распознать цифры в ячейке!')
            result.append(table_result)
        return result
    except Exception as ex:
        logger.warning(f'Ошибка при работе распозновании одной таблицы: {ex}')



if __name__ == '__main__':
    pass



import cv2
import numpy as np
from .setting import logger


def display(im, bbox):
    n = len(bbox)
    for j in range(n):
        cv2.line(im, tuple(bbox[j][0]), tuple(bbox[(j + 1) % n][0]), (255, 0, 0), 3)
    cv2.imshow("Results", im)


def read_qrcode(img):
    try:
        qrDecoder = cv2.QRCodeDetector()
        data, bbox, rectifiedImage = qrDecoder.detectAndDecode(img)
        if len(data) > 0:
            return data
        else:
            return 'cant find qrcode'
    except Exception as ex:
        logger.warning(f'Ошибка расопзновании QR кода: {ex}')

if __name__ == '__main__':
    inputImage = cv2.imread("data/v6.jpg")
    qrDecoder = cv2.QRCodeDetector()
    data, bbox, rectifiedImage = qrDecoder.detectAndDecode(inputImage)
    if len(data) > 0:
        print("Decoded Data : {}".format(data))
        display(inputImage, bbox)
        rectifiedImage = np.uint8(rectifiedImage);
        cv2.imshow("Rectified QRCode", rectifiedImage);
    else:
        print("QR Code not detected")
        cv2.imshow("Results", inputImage)
    cv2.waitKey(0)
    cv2.destroyAllWindows()



import cv2
from .read_cell import make_it
from .qr_table1_table2 import take_it
from .preprocessing import clever_resize
from .paint_orign import paint_marks
from .read_qrcode import read_qrcode
from .setting import logger
import numpy as np

def check_test(image):
    try:
        if type(image) != type(np.array([])):
            image = np.array(image)
        qr_code = read_qrcode(image)
        qr_code = 'v9x10_blankinfo_childinfo' #
        blank_type = qr_code.split('_')[0]
        image, scans, rects = take_it(image, blank_type)
        result = make_it(scans, blank_type)
        logger.info('Распознование проведено!')
        return result, scans, rects, blank_type, qr_code
    except Exception as ex:
        logger.warning(f'Ошибка при проверке бланка {ex}')


def paint(image, all_marks, rects, blank_type):
    if isinstance(image, np.ndarray):
        image = np.array(image)
    for i, marks in enumerate(all_marks):
        image = paint_marks(image, rects[i], marks, blank_type)
    logger.info('Разметка проведена!')
    return image

def sumquadro(a,b):
    return 4*(a + b)


if __name__ == '__main__':
    # параметр для сканируемого изображения
    args_image = 'data/v9.jpg'
    # прочитать изображение
    image = cv2.imread(args_image)
    image = clever_resize(image)
    result, scans, rects, blank_type, qr_code = check_test(image)
    marks1 = [0, 1, 1, 1, 1, 1, 1, 1, 1, 1]
    marks2 = [0, 1, 0, 1, 1, 0, 1, 0, 0, 1]
    all_marks = [marks1, marks2]
    painted_image = paint(image, all_marks, rects, blank_type)
    cv2.imshow('painted_img', painted_image)
    print(result)
    print(qr_code)
    cv2.waitKey(0)  # press 0 to close all cv2 windows
    cv2.destroyAllWindows()
































# # if __name__ == '__main__':
#
# #     #make_it('PyCharm')



import cv2
from skimage.filters import threshold_local


def img_resize(image, scale_percent):
    width = int(image.shape[1] * scale_percent / 100)
    height = int(image.shape[0] * scale_percent / 100)
    dim = (width, height)
    image = cv2.resize(image, dim, interpolation=cv2.INTER_AREA)
    return image


def clever_resize(image):
    if image.shape[1] >= image.shape[0] >= 1300:
        scale_percent = 100000 // image.shape[1]
        return img_resize(image, scale_percent)
    elif image.shape[0] > image.shape[1] > 1300:
        scale_percent = 100000 // image.shape[0]
        return img_resize(image, scale_percent)
    return image


def make_contrast(img):
    th = threshold_local(img, 9, offset=10, method="gaussian")
    contrast_img = (img > th).astype("uint8") * 255
    return contrast_img


if __name__ == '__main__':
    pass



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.image as mpimg
import seaborn as sns
# %matplotlib inline

np.random.seed(2)

from sklearn.model_selection import train_test_split
from sklearn.metrics import confusion_matrix
import itertools

from keras.utils.np_utils import to_categorical # convert to one-hot-encoding
from keras.models import Sequential
from keras.layers import Dense, Dropout, Flatten, Conv2D, MaxPool2D
from keras.optimizers import RMSprop
from keras.preprocessing.image import ImageDataGenerator
from keras.callbacks import ReduceLROnPlateau


sns.set(style='white', context='notebook', palette='deep')

train = pd.read_csv("input/train.csv")
test = pd.read_csv("input/test.csv")

Y_train = train["label"]

# Drop 'label' column
X_train = train.drop(labels = ["label"],axis = 1)

# free some space
del train

g = sns.countplot(Y_train)

print(Y_train.value_counts())

# Check the data
print(X_train.isnull().any().describe())

print(test.isnull().any().describe())


# Normalize the data
X_train = X_train / 255.0
test = test / 255.0

# Reshape image in 3 dimensions (height = 28px, width = 28px , canal = 1)
X_train = X_train.values.reshape(-1,28,28,1)
test = test.values.reshape(-1,28,28,1)

# Encode labels to one hot vectors (ex : 2 -> [0,0,1,0,0,0,0,0,0,0])
Y_train = to_categorical(Y_train, num_classes = 10)

# Set the random seed
random_seed = 2

# Split the train and the validation set for the fitting
X_train, X_val, Y_train, Y_val = train_test_split(X_train, Y_train, test_size = 0.1, random_state=random_seed)


# Set the CNN model
# my CNN architechture is In -> [[Conv2D->relu]*2 -> MaxPool2D -> Dropout]*2 -> Flatten -> Dense -> Dropout -> Out

model = Sequential()

model.add(Conv2D(filters = 32, kernel_size = (5,5),padding = 'Same',
                 activation ='relu', input_shape = (28,28,1)))
model.add(Conv2D(filters = 32, kernel_size = (5,5),padding = 'Same',
                 activation ='relu'))
model.add(MaxPool2D(pool_size=(2,2)))
model.add(Dropout(0.25))


model.add(Conv2D(filters = 64, kernel_size = (3,3),padding = 'Same',
                 activation ='relu'))
model.add(Conv2D(filters = 64, kernel_size = (3,3),padding = 'Same',
                 activation ='relu'))
model.add(MaxPool2D(pool_size=(2,2), strides=(2,2)))
model.add(Dropout(0.25))


model.add(Flatten())
model.add(Dense(256, activation = "relu"))
model.add(Dropout(0.5))
model.add(Dense(10, activation = "softmax"))

# Define the optimizer
optimizer = RMSprop(lr=0.0005, rho=0.9, epsilon=1e-08, decay=0.0)

# Compile the model
model.compile(optimizer = optimizer , loss = "categorical_crossentropy", metrics=["accuracy"])

# Set a learning rate annealer
learning_rate_reduction = ReduceLROnPlateau(monitor='val_acc',
                                            patience=3,
                                            verbose=1,
                                            factor=0.5,
                                            min_lr=0.00001)

epochs = 10 # Turn epochs to 30 to get 0.9967 accuracy
batch_size = 86

# Without data augmentation i obtained an accuracy of 0.98114
history = model.fit(X_train, Y_train, batch_size = batch_size, epochs = epochs,
         validation_data = (X_val, Y_val), verbose = 1)

model.save('my_model_1.h5')
# With data augmentation to prevent overfitting (accuracy 0.99286)

# datagen = ImageDataGenerator(
#         featurewise_center=False,  # set input mean to 0 over the dataset
#         samplewise_center=False,  # set each sample mean to 0
#         featurewise_std_normalization=False,  # divide inputs by std of the dataset
#         samplewise_std_normalization=False,  # divide each input by its std
#         zca_whitening=False,  # apply ZCA whitening
#         rotation_range=10,  # randomly rotate images in the range (degrees, 0 to 180)
#         zoom_range = 0.1, # Randomly zoom image
#         width_shift_range=0.1,  # randomly shift images horizontally (fraction of total width)
#         height_shift_range=0.1,  # randomly shift images vertically (fraction of total height)
#         horizontal_flip=False,  # randomly flip images
#         vertical_flip=False)  # randomly flip images
#
#
# datagen.fit(X_train)

# Fit the model
# history = model.fit_generator(datagen.flow(X_train,Y_train, batch_size=batch_size),
#                               epochs = epochs, validation_data = (X_val,Y_val),
#                               verbose = 2, steps_per_epoch=X_train.shape[0] // batch_size
#                               , callbacks=[learning_rate_reduction])


from setuptools import setup


setup(
    name='mentalka_test_checker',                       # package name
    version='0.2',                                      # version
    author="Salim",
    author_email="cvwisework@gmail.com",
    description='OCR system for table-test check',      # short description
    long_description='''OCR system for table-test check. Use 
    result, scans, rects, blank_type, qr_code = check_test(image)
    painted_image = paint(image, all_marks, rects, blank_type)
    ''',
    long_description_content_type = 'text/x-rst',
    url='https://github.com/SalimMaxHigh/test_checker',                           # package URL
    install_requires=['h5py==2.10.0',
                      'scipy==1.4.1',
                      'opencv-python',
                      'numpy==1.18.5',
                      'scikit-image==0.17.2',
                      'imutils==0.5.3',
                      'tensorflow==2.3',
                      'Keras==2.4.3'],                # list of packages this package depends
                                                        # on.
    packages=['mentalka_test_checker'],                                   # List of module names that installing
                                                        # this package will provide.
) 



import cv2
from .setting import mark_size, correct_mark_color, wrong_mark_color, mark_thickness
from .setting import logger


def paint_marks(image, rect, marks, blank_type):
    try:
        (tl, tr, br, bl) = rect
        w = br[0] - bl[0]
        hl = bl[1] - tl[1]
        hr = br[1] - tr[1]
        for i in range(len(marks)):
            w_ = ((4 + 5 * i) * w / 52)
            xm = bl[0] + w_
            ymm = (((1 * hl / 16) * (w - w_) + (1 * hr / 16) * w_) / w)
            ym = ((bl[1] * (w - w_) + br[1] * w_) / w) - ymm
            x_new, y_new = int(xm), int(ym)
            if marks[i] == 0:
                image = cv2.line(image, (x_new - mark_size, y_new), (x_new + mark_size, y_new), wrong_mark_color, mark_thickness)
            else:
                image = cv2.line(image, (x_new - mark_size, y_new), (x_new + mark_size, y_new), correct_mark_color, mark_thickness)
                image = cv2.line(image, (x_new, y_new + mark_size), (x_new, y_new - mark_size), correct_mark_color, mark_thickness)
        return image
    except Exception as ex:
        logger.warning(f'Ошибка при наложении результатов проверки на изображение: {ex}')



from keras.models import load_model
import cv2
import numpy as np
from .setting import DONT_SHOW, logger, BASE_DIR
import time
import os

model_path = os.path.join(BASE_DIR, 'model/model_mnist.h5')
model = load_model(model_path)


def predict_img(img):
    try:
        width = int(28)
        height = int(28)
        dim = (width, height)
        img = cv2.resize(img, dim, interpolation=cv2.INTER_AREA)
        if not DONT_SHOW:
            time.sleep(0.001)
            cv2.imwrite(f'write/v_{str(int(time.time() * 1000))}.jpg', img)
        if np.sum(img) > 188000:
            return '_'
        else:
            img = 255 - img
            img = img / 255
            if not DONT_SHOW:
                cv2.imshow(f'ff{4}', img)
            img = img.reshape(1, 28, 28, 1)
            return str(np.argmax( model.predict([img])))
    except Exception as ex:
        logger.warning(f'Ошибка при предсказании числа: {ex}')